<?php

function w2dc_current_user_can_edit_review($review_id) {
	if (!current_user_can('edit_others_posts')) {
		$post = get_post($review_id);
		$current_user = wp_get_current_user();
		if ($current_user->ID != $post->post_author)
			return false;
		if ($post->post_status == 'pending'  && !is_admin())
			return false;
	}
	return true;
}

function w2dc_show_edit_review_button($review_id) {
	global $w2dc_instance;
	if (
		w2dc_current_user_can_edit_review($review_id)
		&&
		(
			(get_option('w2dc_fsubmit_addon') && isset($w2dc_instance->dashboard_page_url) && $w2dc_instance->dashboard_page_url)
			||
			((!get_option('w2dc_fsubmit_addon') || !isset($w2dc_instance->dashboard_page_url) || !$w2dc_instance->dashboard_page_url) && !get_option('w2dc_hide_admin_bar') && current_user_can('edit_posts'))
		)
	)
		return true;
}

function w2dc_get_edit_review_link($review_id, $context = 'display') {
	if (w2dc_current_user_can_edit_review($review_id)) {
		$post = get_post($review_id);
		$current_user = wp_get_current_user();
		if (current_user_can('edit_others_posts') && $current_user->ID != $post->post_author) {
			return get_edit_post_link($review_id, $context);
		} else {
			return apply_filters('w2dc_get_edit_review_link', get_edit_post_link($review_id, $context), $review_id);
		}
	}
}

function w2dc_getAddReviewPage() {
	global $wpdb, $w2dc_instance;

	if ($pages = $wpdb->get_results("SELECT ID AS id FROM {$wpdb->posts} WHERE post_content LIKE '%[" . W2DC_ADD_REVIEW_PAGE_SHORTCODE . "]%' AND post_status = 'publish' AND post_type = 'page'", ARRAY_A)) {
		$page_id = $pages[0]['id'];

		// adapted for WPML
		global $sitepress;
		if (function_exists('wpml_object_id_filter') && $sitepress) {
			if ($tpage = apply_filters('wpml_object_id', $page_id, 'page')) {
				$page_id = $tpage;
			}

		}

		return $page_id;
	}
}

function w2dc_get_add_review_link($listing_id) {
	global $wp_rewrite;
	
	$listing_url = get_permalink($listing_id);
	
	if ($wp_rewrite->using_permalinks()) {
		return rtrim($listing_url, '/') . '/add-review/';
	} else {
		if (!($add_review_page_id = w2dc_getAddReviewPage())) {
			$add_review_page_url = $listing_url;
		} else {
			$add_review_page_url = get_the_permalink($add_review_page_id);
		}
		$listing = w2dc_getListing($listing_id);
		
		return add_query_arg(array('listing-w2dc' => $listing->post->post_name, 'review-action-w2dc' => 'new'), $add_review_page_url);
	}
}

function w2dc_can_user_add_review($listing_id) {
	global $wpdb;
	
	$reviews_counter = 0;
	if ($current_user = wp_get_current_user()) {
		$reviews_counter = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON p.ID=pm.post_id WHERE p.post_type='" . W2DC_REVIEW_TYPE . "' AND (p.post_status='publish' OR p.post_status='pending') AND p.post_author={$current_user->ID} AND pm.meta_key='_listing_id' AND pm.meta_value={$listing_id}");
	}
	
	if (
		current_user_can('manage_options') ||
		!is_user_logged_in() ||
		!get_option('w2dc_reviews_number_allowed') ||
		(get_option('w2dc_reviews_allowed_users') == 'login' && is_user_logged_in() && $reviews_counter < get_option('w2dc_reviews_number_allowed')) ||
		get_option('w2dc_reviews_allowed_users') == 'guests' ||
		get_option('w2dc_reviews_allowed_users') == 'no_contact_form'
	) {
		return true;
	}
}

function w2dc_get_all_reviews_link($listing) {
	global $wp_rewrite;
	
	$listing_url = get_permalink($listing->post->ID);
	
	if ($wp_rewrite->using_permalinks()) {
		return rtrim($listing_url, '/') . '/reviews/';
	} else {
		return add_query_arg('review-action-w2dc', 'all', $listing_url);
	}
}

function w2dc_getReview($post) {
	$review = new w2dc_review;
	if ($review->loadReviewFromPost($post)) {
		return $review;
	}
}

function w2dc_getReviewPage() {
	global $wpdb;

	if ($pages = $wpdb->get_results("SELECT ID AS id FROM {$wpdb->posts} WHERE post_content LIKE '%[" . W2DC_REVIEW_PAGE_SHORTCODE . "]%' AND post_status = 'publish' AND post_type = 'page'", ARRAY_A)) {
		$page_id = $pages[0]['id'];
	
		// adapted for WPML
		global $sitepress;
		if (function_exists('wpml_object_id_filter') && $sitepress) {
			if ($tpage = apply_filters('wpml_object_id', $page_id, 'page')) {
				$page_id = $tpage;
			}
	
		}
	
		return $page_id;
	}
}

function w2dc_reviewsOrderLinks($base_url, $defaults = array(), $return = false, $shortcode_hash = null) {
	global $w2dc_instance;

	if (isset($_GET['reviews_order_by']) && $_GET['reviews_order_by']) {
		$order_by = $_GET['reviews_order_by'];
		$order = w2dc_getValue($_GET, 'reviews_order', 'ASC');
	} else {
		if (isset($defaults['reviews_order_by']) && $defaults['reviews_order_by']) {
			$order_by = $defaults['reviews_order_by'];
			$order = w2dc_getValue($defaults, 'reviews_order', 'ASC');
		} else {
			$order_by = 'post_date';
			$order = 'DESC';
		}
	}

	$ordering = array();
	$ordering['post_date']['DESC'] = __('Newest first', 'W2DC');
	$ordering['post_date']['ASC'] = __('Oldest first', 'W2DC');
	$ordering['rating'] = __('Best rating', 'W2DC');
	if (get_option('w2dc_reviews_votes')) {
		$ordering['votes'] = __('Best voted', 'W2DC');
	}
	
	$ordering_links = new w2dc_orderingLinks($ordering, $base_url, $order_by, $order);
	
	$ordering_links = apply_filters('w2dc_reviews_ordering_options', $ordering_links, $base_url, $defaults, $shortcode_hash);
	
	return $ordering_links;
}

function w2dc_reviewsOrderingItems() {
	global $w2dc_instance;

	$ordering = array('post_date' => __('Date', 'W2DC'), 'votes' => __('Votes', 'W2DC'), 'rating' => __('Rating', 'W2DC'));

	$ordering = apply_filters('w2dc_default_reviews_orderby_options', $ordering);
	$ordering_items = array();
	foreach ($ordering AS $field_slug=>$field_name) {
		$ordering_items[] = array('value' => $field_slug, 'label' => $field_name);
	}

	return $ordering_items;
}

?>