<?php 

class w2dc_reviews_controller extends w2dc_frontend_controller {
	public $request_by = 'reviews_controller';
	public $reviews = array();
	public $review;
	public $head_post;

	public function init($args = array()) {
		global $w2dc_instance;
		
		parent::init($args);
	
		if (get_query_var('page')) {
			$paged = get_query_var('page');
		} elseif (get_query_var('paged')) {
			$paged = get_query_var('paged');
		} else {
			$paged = 1;
		}
		
		$shortcode_atts = array_merge(array(
				'listings' => '',
				'name' => '',
				'perpage' => 10,
				'onepage' => 0,
				'reviews_order_by' => 'post_date',
				'reviews_order' => 'DESC',
				'status' => 'publish',
				//'hide_order' => 0,
				//'hide_count' => 0,
				//'hide_paginator' => 0,
				//'show_views_switcher' => 1,
				//'reviews_view_type' => 'list',
				//'reviews_view_grid_columns' => 2,
				//'reviews_thumb_width' => (int)get_option('w2dc_listing_thumb_width'),
				//'wrap_logo_list_view' => 0,
				//'logo_animation_effect' => (int)get_option('w2dc_logo_animation_effect'),
				//'hide_content' => 0,
				//'summary_on_logo_hover' => 0,
				'author' => 0,
				'paged' => $paged,
				//'scrolling_paginator' => 0,
				//'grid_view_logo_ratio' => get_option('w2dc_grid_view_logo_ratio'), // 100 (1:1), 75 (4:3), 56.25 (16:9), 50 (2:1)
				'include_get_params' => 1,
				'template' => array(W2DC_RATINGS_TEMPLATES_PATH, 'reviews_block.tpl.php'),
				'uid' => null,
		), $args);
		$shortcode_atts = apply_filters('w2dc_related_shortcode_args', $shortcode_atts, $args);
		
		$this->args = $shortcode_atts;
		if ($shortcode_atts['include_get_params']) {
			$get_params = $_GET;
			array_walk_recursive($get_params, 'sanitize_text_field');
			$this->args = array_merge($this->args, $get_params);
		}

		$base_url_args = apply_filters('w2dc_base_url_args', array());
		if (isset($args['base_url'])) {
			$this->base_url = add_query_arg($base_url_args, $args['base_url']);
		} else {
			$this->base_url = add_query_arg($base_url_args, get_permalink());
		}

		$this->template = $this->args['template'];

		$args = array(
				'post_type' => W2DC_REVIEW_TYPE,
				'post_status' => $shortcode_atts['status'],
				'posts_per_page' => $shortcode_atts['perpage'],
				'paged' => $paged,
		);
		if ($shortcode_atts['name']) {
			$args['name'] = $shortcode_atts['name'];
		}
		if ($shortcode_atts['listings']) {
			if (is_string($shortcode_atts['listings'])) {
				$listings = explode(',', $shortcode_atts['listings']);
			} elseif (is_array($shortcode_atts['listings'])) {
				$listings = $shortcode_atts['listings'];
			}
			$args['meta_query'][] = array(
					'key' => '_listing_id',
					'value' => $listings,
					'compare' => 'IN'
			);
		}
		if ($shortcode_atts['author']) {
			$args['author'] = $shortcode_atts['author'];
		}

		// render just one page - all found listings on one page
		if ($shortcode_atts['onepage']) {
			$args['posts_per_page'] = -1;
		}

		$args = array_merge($args, apply_filters('w2dc_reviews_order_args', array(), $shortcode_atts, $this->args['include_get_params']));

		if (!empty($this->args['post__in'])) {
			if (is_string($this->args['post__in'])) {
				$args = array_merge($args, array('post__in' => explode(',', $this->args['post__in'])));
			} elseif (is_array($this->args['post__in'])) {
				$args['post__in'] = $this->args['post__in'];
			}
		}
		if (!empty($this->args['post__not_in'])) {
			$args = array_merge($args, array('post__not_in' => explode(',', $this->args['post__not_in'])));
		}
		
		$this->query = new WP_Query($args);
		//var_dump($this->query->request);
		
		while ($this->query->have_posts()) {
			$this->query->the_post();
		
			$review = new w2dc_review;
			$review->loadReviewFromPost(get_post());
		
			$this->reviews[get_the_ID()] = $review;
		}
		
		// this is reset is really required after the loop ends
		wp_reset_postdata();
		
		apply_filters('w2dc_reviews_controller_construct', $this);
	}
	
	public function getReviewsBlockClasses() {
		$classes[] = "w2dc-container-fluid";
		$classes[] = "w2dc-reviews-block";
		
		$classes = apply_filters("w2dc_reviews_block_classes", $classes, $this);
	
		return implode(" ", $classes);
	}
	
	public function getReviewClasses() {
		$classes = array();

		return $classes;
	}
}

add_filter('w2dc_reviews_order_args', 'w2dc_order_reviews', 10, 3);
function w2dc_order_reviews($order_args = array(), $defaults = array(), $include_GET_params = true) {
	global $w2dc_instance;

	if ($include_GET_params && isset($_GET['reviews_order_by']) && $_GET['reviews_order_by']) {
		$order_by = $_GET['reviews_order_by'];
		$order = w2dc_getValue($_GET, 'reviews_order', 'DESC');
	} else {
		if (isset($defaults['reviews_order_by']) && $defaults['reviews_order_by']) {
			$order_by = $defaults['reviews_order_by'];
			$order = w2dc_getValue($defaults, 'reviews_order', 'DESC');
		} else {
			$order_by = 'post_date';
			$order = 'DESC';
		}
	}

	$order_args['orderby'] = $order_by;
	$order_args['order'] = $order;

	if ($order_by == 'post_date') {
		$order_args['orderby'] = 'date';
	} elseif ($order_by == 'rating') {
		$order_args['orderby'] = 'meta_value_num';
		$order_args['meta_key'] = '_avg_rating';
	} elseif ($order_by == 'votes') {
		$order_args['orderby'] = 'meta_value_num';
		$order_args['meta_key'] = '_votes_sum';
	}

	return $order_args;
}

?>