<?php

class w2dc_reviews_manager {
	
	public function __construct() {
		include_once W2DC_RATINGS_PATH . 'classes/reviews_widget.php';
		
		add_action('add_meta_boxes', array($this, 'addListingMetabox'));
		add_action('add_meta_boxes', array($this, 'addMediaMetabox'));
		if (get_option('w2dc_reviews_multi_rating')) {
			add_action('add_meta_boxes', array($this, 'addRatingsCriteriasMetabox'));
		} else {
			add_action('add_meta_boxes', array($this, 'addRatingMetabox'));
		}
		
		add_filter('postbox_classes_' . W2DC_REVIEW_TYPE . '_w2dc_review_listing', array($this, 'addMetaboxClasses'));
		
		add_filter('manage_'.W2DC_REVIEW_TYPE.'_posts_columns', array($this, 'add_reviews_table_columns'));
		add_filter('manage_'.W2DC_REVIEW_TYPE.'_posts_custom_column', array($this, 'manage_reviews_table_rows'), 10, 2);
		
		// do not allow Post Type Switcher plugin to break attachment post
		add_filter('pts_allowed_pages', array($this, 'avoid_post_type_switcher'));
		
		if ((isset($_POST['publish']) || isset($_POST['save'])) && (isset($_POST['post_type']) && $_POST['post_type'] == W2DC_REVIEW_TYPE)) {
			// the post with empty title will not call this filter!
			add_filter('wp_insert_post_data', array($this, 'validateReview'), 99, 2);

			add_action('save_post_' . W2DC_REVIEW_TYPE, array($this, 'saveReview'), 10, 3);
		}
		
		add_filter('w2dc_do_use_comments_template', '__return_false');
		add_action('w2dc_comments_system', array($this, 'display_reviews'));
		add_filter('w2dc_comments_label', array($this, 'comments_label'), 10, 2);
		add_filter('w2dc_comments_reply_label', array($this, 'comments_reply_label'), 10, 2);
			
		add_filter('w2dc_do_process_directory_controller', array($this, 'do_process_directory_controller'));
		add_filter('w2dc_directory_controller_construct', array($this, 'directory_controller_construct'));
		add_filter('w2dc_get_directory_controller', array($this, 'get_review_controller'));
			
		add_action('wp_ajax_w2dc_review_vote', array($this, 'review_vote'));
		add_action('wp_ajax_nopriv_w2dc_review_vote', array($this, 'review_vote'));
			
		add_filter('query_vars', array($this, 'add_query_vars'));
		add_action('w2dc_build_rules', array($this, 'build_rules'), 10, 4);
			
		add_action('w2dc_login_registration_pages', array($this, 'login_registration_pages'));
			
		add_action('transition_post_status', array($this, 'on_review_approval'), 10, 3);
		
		add_action('trashed_post', array($this, 'delete_review_ratings'));
		add_action('delete_post', array($this, 'delete_review_ratings'));
		add_action('untrashed_post', array($this, 'restore_review_ratings'));
		
		add_action('w2dc_reset_ratings', array($this, 'updateListingRatings'));
		
		add_action('w2dc_dashboard_links', array($this, 'add_reviews_dashboard_link'));
		add_filter('w2dc_dashboard_controller_construct', array($this, 'handle_dashboard_controller'));
		add_filter('w2dc_get_edit_review_link', array($this, 'edit_review_link'), 10, 2);
		
		add_filter('w2dc_count_attachments', array($this, 'count_attachments'), 10, 2);
		
		add_action('wp_ajax_w2dc_reviews_controller_request', array($this, 'controller_request'));
		add_action('wp_ajax_nopriv_w2dc_reviews_controller_request', array($this, 'controller_request'));
	}

	public function addMetaboxClasses($classes = array()) {
		$classes[] = 'w2dc-sidebar-metabox';
		
		return $classes;
	}
	
	public function addListingMetabox($post_type) {
		if ($post_type == W2DC_REVIEW_TYPE) {
			add_meta_box('w2dc_review_listing',
			__('Review for a listing', 'W2DC'),
			array($this, 'listingMetabox'),
			W2DC_REVIEW_TYPE,
			'side',
			'core');
		}
	}
	
	public function addRatingMetabox($post_type) {
		if ($post_type == W2DC_REVIEW_TYPE) {
			add_meta_box('w2dc_rating',
			__('Review rating', 'W2DC'),
			array($this, 'ratingMetabox'),
			W2DC_REVIEW_TYPE,
			'normal',
			'high');
		}
	}
	
	public function ratingMetabox($post) {
		$review = w2dc_getReview($post);
		
		w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'avg_rating.tpl.php'), array('avg_rating' => $review->avg_rating, 'post_id' => $review->post->ID, 'noajax' => true, 'meta_tags' => false, 'active' => true, 'show_avg' => false));
	}
	
	public function addRatingsCriteriasMetabox($post_type) {
		if ($post_type == W2DC_REVIEW_TYPE) {
			add_meta_box('w2dc_ratings_criterias',
			__('Review ratings', 'W2DC'),
			array($this, 'ratingsCriteriasMetabox'),
			W2DC_REVIEW_TYPE,
			'normal',
			'high');
		}
	}
	
	public function addMediaMetabox($post_type) {
		global $w2dc_instance, $post;
		
		if ($post_type == W2DC_REVIEW_TYPE) {
			if (get_option("w2dc_reviews_images_number")) {
				$review = w2dc_getReview($post);
				$w2dc_instance->media_manager->load_media(array(
						'images' => $review->images,
						//'logo_image' => $listing->logo_image,
				));
				$w2dc_instance->media_manager->load_params(array(
						'object_id' => $post->ID,
						'images_number' => get_option("w2dc_reviews_images_number"),
						'videos_number' => 0,
						'logo_enabled' => false,
				));
	
				add_action('admin_enqueue_scripts', array($w2dc_instance->media_manager, 'admin_enqueue_scripts_styles'));
					
				add_meta_box('w2dc_media_metabox',
				__('Review media', 'W2DC'),
				array($w2dc_instance->media_manager, 'mediaMetabox'),
				W2DC_REVIEW_TYPE,
				'normal',
				'high',
				array('target' => 'reviews'));
			}
		}
	}
	
	public function count_attachments($do_upload, $post_id) {
		if ($review = w2dc_getReview($post_id)) {
			$existed_images_count = count($review->images);
			
			if ($existed_images_count >= get_option("w2dc_reviews_images_number")) {
				$do_upload = false;
			}
		}
	
		return $do_upload;
	}
	
	public function listingMetabox($post) {
		global $w2dc_instance;
		
		$listings = get_posts(
				array(
						'post_type'   => W2DC_POST_TYPE,
						'orderby'     => 'title',
						'order'       => 'ASC',
						'numberposts' => -1
				)
		);
	
		$listing_id = false;
		$listing = false;
		if ($listing_id = get_post_meta($post->ID, '_listing_id', true)) {
			if (w2dc_getListing($listing_id)) {
				$listing = w2dc_getListing($listing_id);
			}
		}
		
		w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'review_listing_metabox.tpl.php'), array('listings' => $listings, 'selected_listing_id' => $listing_id, 'listing' => $listing));
	}
	
	public function getRatingsCriterias() {
		$ratings_criterias = array();
		if (get_option('w2dc_reviews_multi_rating')) {
			$ratings_criterias = explode('
', get_option('w2dc_reviews_multi_rating'));
		}
		return array_filter($ratings_criterias);
	}
	
	public function ratingsCriteriasMetabox($post) {
		$review_ratings = get_post_meta($post->ID, '_review_ratings', true);
		
		$ratings_criterias = $this->getRatingsCriterias();
		
		w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'review_ratings_metabox.tpl.php'), array('review_ratings' => $review_ratings, 'ratings_criterias' => $ratings_criterias));
	}
	
	public function add_reviews_table_columns($columns) {
		global $w2dc_instance;
	
		$w2dc_columns['w2dc_rating'] = __('Rating', 'W2DC');
		$w2dc_columns['w2dc_listing'] = __('Listing', 'W2DC');

		return array_slice($columns, 0, 2, true) + $w2dc_columns + array_slice($columns, 2, count($columns)-2, true);
	}
	
	public function manage_reviews_table_rows($column, $post_id) {
		switch ($column) {
			case "w2dc_listing":
				if ($listing_id = get_post_meta($post_id, '_listing_id', true)) {
					if ($listing = w2dc_getListing($listing_id)) {
						echo edit_post_link($listing->title(), '', '', $listing);
					}
				}
				break;
			case "w2dc_rating":
				$review = w2dc_getReview($post_id);
				$review->renderStars();
				break;
		}
	}
	
	public function saveInitialDraft($post_id, $listing) {
		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
			return;
	
		add_post_meta($post_id, '_listing_id', $listing->post->ID);
	}
	
	public function validateReview($data, $postarr) {
		if ($data['post_type'] == W2DC_REVIEW_TYPE) {
			global $w2dc_instance, $post;
	
			if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
				return;
	
			$errors = array();
				
			if (empty($postarr['w2dc_listing_id'])) {
				$errors[] = __('Select listing for review!', 'W2DC');
			}
	
			if (get_option("w2dc_reviews_images_number")) {
				$w2dc_instance->media_manager->load_params(array(
						'object_id' => $post->ID,
						'images_number' => get_option("w2dc_reviews_images_number"),
						'videos_number' => 0,
						'logo_enabled' => false,
				));
				
				if ($validation_results = $w2dc_instance->media_manager->validateAttachments($errors)) {
					$w2dc_instance->media_manager->saveAttachments($validation_results);
				}
			}
	
			// only successfully validated listings can be completed
			if ($errors) {
				foreach ($errors AS $error) {
					w2dc_addMessage($error, 'error');
				}
			} else {
				w2dc_addMessage(__('Review was saved successfully!', 'W2DC'));
			}
		}
		return $data;
	}
	
	public function getReviewsOfListing($listing_id) {
		$args = array(
				'post_type' => W2DC_REVIEW_TYPE,
				'post_status' => 'publish',
				'meta_query' => array(
						array(
						'key' => '_listing_id',
						'value' => $listing_id
						)
				)
		);
		$reviews_posts = get_posts($args);
		return $reviews_posts;
	}
	
	public function saveReviewRatings($review_id) {
		if ($ratings_criterias = $this->getRatingsCriterias()) {
			$total_rating = 0;
			foreach ($ratings_criterias AS $key=>$criteria) {
				$rating = w2dc_getValue($_POST, 'w2dc_review_rating_' . $key, 9);
				$review_ratings[$key] = $rating;
				$total_rating += $rating;
			}
			$avg_rating = ($total_rating/count($ratings_criterias)+1)/2;
	
			update_post_meta($review_id, '_review_ratings', $review_ratings);
			update_post_meta($review_id, '_avg_rating', $avg_rating);
		} else {
			$rating = w2dc_getValue($_POST, 'w2dc-rating-noajax-' . $review_id, 5);
	
			update_post_meta($review_id, '_avg_rating', $rating);
		}
	}
	
	public function saveAllRatings($review_id) {
		global $w2dc_ratings_instance;
		
		$listing_id = get_post_meta($review_id, '_listing_id', true);
		
		if ($ratings_criterias = $this->getRatingsCriterias()) {
			$total_rating = 0;
			foreach ($ratings_criterias AS $key=>$criteria) {
				$rating = w2dc_getValue($_POST, 'w2dc_review_rating_' . $key, 9);
				$review_ratings[$key] = $rating;
				$total_rating += $rating;
			}
			$avg_rating = ($total_rating/count($ratings_criterias)+1)/2;
			
			$w2dc_ratings_instance->save_rating($listing_id, $avg_rating, $review_id);
		} else {
			$rating = w2dc_getValue($_POST, 'w2dc-rating-noajax-' . $review_id, 5);
			
			$w2dc_ratings_instance->save_rating($listing_id, $rating, $review_id);
		}
		
		$this->saveReviewRatings($review_id);
		
		$this->updateListingRatings($listing_id);
	}
	
	public function updateListingRatings($listing_id) {
		global $w2dc_ratings_instance, $wpdb;
		
		// collect all active reviews of the target post
		$active_reviews_ids = array();
		if ($reviews_posts = $this->getReviewsOfListing($listing_id)) {
			foreach ($reviews_posts AS $post) {
				$review = $post;
				$active_reviews_ids[] = $review->ID;
			}
		}
		
		// delete unnecessary reviews ratings from target post's meta
		$like = $wpdb->esc_like(W2DC_RATING_PREFIX . 'review_');
		$results = $wpdb->get_results($wpdb->prepare("SELECT meta_id, meta_key FROM {$wpdb->postmeta} WHERE post_id=%d AND meta_key LIKE %s", $post_id, $like.'%'), ARRAY_A);
		foreach ($results AS $row) {
			$review_id = str_replace(W2DC_RATING_PREFIX . 'review_', '', $row['meta_key']);
				
			if (!in_array($review_id, $active_reviews_ids)) {
				delete_post_meta($listing_id, $row['meta_key']);
			}
		}
		
		// update average rating
		foreach ($active_reviews_ids AS $review_id) {
			$avg_rating = get_post_meta($review_id, '_avg_rating', true);
				
			$w2dc_ratings_instance->save_rating($listing_id, $avg_rating, $review_id);
		}
	}
	
	public function delete_review_ratings($post_id) {
		if (get_post_type($post_id) == W2DC_REVIEW_TYPE) {
			$review = w2dc_getReview($post_id);
			$review_id = $review->post->ID;
			$listing_id = $review->listing->post->ID;
			
			delete_post_meta($listing_id, W2DC_RATING_PREFIX . 'review_' . $review_id);
			
			$this->updateListingRatings($listing_id);
		}
	}
	
	public function restore_review_ratings($post_id) {
		if (get_post_type($post_id) == W2DC_REVIEW_TYPE) {
			$review = w2dc_getReview($post_id);
			$listing_id = $review->listing->post->ID;
			
			$this->updateListingRatings($listing_id);
		}
	}
	
	public function saveReviewMeta($post_ID) {
		add_post_meta($post_ID, '_review_created', true);
		add_post_meta($post_ID, '_order_date', time());
		add_post_meta($post_ID, '_votes_sum', 0);
	}
	
	public function saveReview($post_ID, $post, $update) {
		if (!empty($_POST['w2dc_listing_id'])) {
			$listing_id = $_POST['w2dc_listing_id'];
			
			update_post_meta($post_ID, '_listing_id', $listing_id);
		
			$this->saveAllRatings($post_ID);
			
			$this->saveReviewMeta($post_ID);
		}
	}
	
	public function avoid_post_type_switcher($pages) {
		if (w2dc_getValue($_POST, 'post_type') == W2DC_REVIEW_TYPE) {
			return;
		}
		return $pages;
	}
	
	public function get_reviews_counter($listing) {
		global $wpdb;
		
		$reviews_counter = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->posts} AS p LEFT JOIN {$wpdb->postmeta} AS pm ON p.ID = pm.post_ID WHERE p.post_type='" . W2DC_REVIEW_TYPE . "' AND p.post_status='publish' AND pm.meta_key='_listing_id' AND pm.meta_value={$listing->post->ID}");
		
		return $reviews_counter;
	}
	
	public function comments_label($label, $listing) {
		$reviews_counter = $this->get_reviews_counter($listing);
		
		return _n('Review', 'Reviews', $reviews_counter, 'W2DC') . ' (' . $reviews_counter . ')';
	}
	
	public function comments_reply_label($label, $listing) {
		$reviews_counter = $this->get_reviews_counter($listing);
		
		return $reviews_counter . ' ' . _n('review', 'reviews', $reviews_counter, 'W2DC');
	}
	
	public function display_reviews($listing) {
		w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'reviews_template.tpl.php'), array('listing' => $listing));
	}
	
	public function add_query_vars($vars) {
		$vars[] = 'review-w2dc';
		$vars[] = 'review-action-w2dc';
	
		return $vars;
	}
	
	public function build_rules($rules, $page_url, $listing_page_id, $directory) {
		global $wp_rewrite;
		
		$listing_slug = $directory->listing_slug;
	
		if (!($review_page_id = w2dc_getReviewPage())) {
			$review_page_id = $listing_page_id;
		}
		if (!($add_review_page_id = w2dc_getAddReviewPage())) {
			$add_review_page_id = $review_page_id;
		}
	
	
		// (.+)reviews/ ---------------------------------------------------------------------------------------------------------
	
		// /%listing_slug%/%category%/%postname%/
		// /%listing_slug%/%location%/%postname%/
		// /%listing_slug%/%tag%/%postname%/
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/(.+?)/([^\/.]+)/reviews/' . $wp_rewrite->pagination_base . '/([0-9]{1,})/?$'] = 'index.php?page_id=' . $review_page_id . '&tax-slugs-w2dc=$matches[2]&listing-w2dc=$matches[3]&paged=$matches[4]&review-action-w2dc=all';
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/(.+?)/([^\/.]+)/reviews/?$'] = 'index.php?page_id=' . $review_page_id . '&tax-slugs-w2dc=$matches[2]&listing-w2dc=$matches[3]&review-action-w2dc=all';
		// /%listing_slug%/%postname%/
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/([^\/.]+)/reviews/' . $wp_rewrite->pagination_base . '/([0-9]{1,})/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[2]&paged=$matches[3]&review-action-w2dc=all';
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/([^\/.]+)/reviews/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[2]&review-action-w2dc=all';
		// /%postname%/
		$rules[$page_url . '/([^\/.]+)/reviews/' . $wp_rewrite->pagination_base . '/([0-9]{1,})/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[1]&paged=$matches[2]&review-action-w2dc=all';
		$rules[$page_url . '/([^\/.]+)/reviews/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[1]&review-action-w2dc=all';
		// /%post_id%/%postname%/
		if (
		strpos(get_option('permalink_structure'), '/%post_id%/%postname%') === FALSE &&
		strpos(get_option('permalink_structure'), '/%year%/%postname%') === FALSE
		) {
			// /%post_id%/%postname%/ will not work when /%post_id%/%postname%/ or /%year%/%postname%/ was enabled for native WP posts
			// also avoid mismatches with archive pages with /%year%/%monthnum%/ permalinks structure
			$rules['(' . $page_url . ')?/?(?!(?:199[0-9]|20[012][0-9])/(?:0[1-9]|1[012]))([0-9]+)/([^\/.]+)/reviews/' . $wp_rewrite->pagination_base . '/([0-9]{1,})/?$'] = 'index.php?page_id=' . $listing_page_id . '&listing-w2dc=$matches[3]&paged=$matches[4]&review-action-w2dc=all';
			$rules['(' . $page_url . ')?/?(?!(?:199[0-9]|20[012][0-9])/(?:0[1-9]|1[012]))([0-9]+)/([^\/.]+)/reviews/?$'] = 'index.php?page_id=' . $listing_page_id . '&listing-w2dc=$matches[3]&review-action-w2dc=all';
		}
	
	
		// (.+)reviews/(.+) ---------------------------------------------------------------------------------------------------------
	
		// /%listing_slug%/%category%/%postname%/
		// /%listing_slug%/%location%/%postname%/
		// /%listing_slug%/%tag%/%postname%/
		$rules['(?!.*/(?:add-review)/?$)(' . $page_url . ')?/?' . $listing_slug . '/(.+?)/([^\/.]+)/reviews/(.+?)/?$'] = 'index.php?page_id=' . $review_page_id . '&tax-slugs-w2dc=$matches[2]&listing-w2dc=$matches[3]&review-w2dc=$matches[4]';
		// /%listing_slug%/%postname%/
		$rules['(?!.*/(?:add-review)/?$)(' . $page_url . ')?/?' . $listing_slug . '/([^\/.]+)/reviews/(.+?)/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[2]&review-w2dc=$matches[3]';
		// /%postname%/
		$rules['(?!.*/(?:add-review)/?$)' . $page_url . '/([^\/.]+)/reviews/(.+?)/?$'] = 'index.php?page_id=' . $review_page_id . '&listing-w2dc=$matches[1]&review-w2dc=$matches[2]';
		// /%post_id%/%postname%/
		if (
		strpos(get_option('permalink_structure'), '/%post_id%/%postname%') === FALSE &&
		strpos(get_option('permalink_structure'), '/%year%/%postname%') === FALSE
		) {
			// /%post_id%/%postname%/ will not work when /%post_id%/%postname%/ or /%year%/%postname%/ was enabled for native WP posts
			// also avoid mismatches with archive pages with /%year%/%monthnum%/ permalinks structure
			$rules['(?!.*/(?:add-review)/?$)(' . $page_url . ')?/?(?!(?:199[0-9]|20[012][0-9])/(?:0[1-9]|1[012]))([0-9]+)/([^\/.]+)/reviews/(.+?)/?$'] = 'index.php?page_id=' . $listing_page_id . '&listing-w2dc=$matches[3]&review-w2dc=$matches[4]';
		}
	
	
		// (.+)add-review/ ---------------------------------------------------------------------------------------------------------
	
		// /%listing_slug%/%category%/%postname%/
		// /%listing_slug%/%location%/%postname%/
		// /%listing_slug%/%tag%/%postname%/
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/(.+?)/([^\/.]+)/add-review/?$'] = 'index.php?page_id=' . $add_review_page_id . '&tax-slugs-w2dc=$matches[2]&listing-w2dc=$matches[3]&review-action-w2dc=new';
		// /%listing_slug%/%postname%/
		$rules['(' . $page_url . ')?/?' . $listing_slug . '/([^\/.]+)/add-review/?$'] = 'index.php?page_id=' . $add_review_page_id . '&listing-w2dc=$matches[2]&review-action-w2dc=new';
		// /%postname%/
		$rules[$page_url . '/([^\/.]+)/add-review/?$'] = 'index.php?page_id=' . $add_review_page_id . '&listing-w2dc=$matches[1]&review-action-w2dc=new';
		// /%post_id%/%postname%/
		if (
		strpos(get_option('permalink_structure'), '/%post_id%/%postname%') === FALSE &&
		strpos(get_option('permalink_structure'), '/%year%/%postname%') === FALSE
		) {
			// /%post_id%/%postname%/ will not work when /%post_id%/%postname%/ or /%year%/%postname%/ was enabled for native WP posts
			// also avoid mismatches with archive pages with /%year%/%monthnum%/ permalinks structure
			$rules['(' . $page_url . ')?/?(?!(?:199[0-9]|20[012][0-9])/(?:0[1-9]|1[012]))([0-9]+)/([^\/.]+)/add-review/?$'] = 'index.php?page_id=' . $add_review_page_id . '&listing-w2dc=$matches[3]&review-action-w2dc=new';
		}
	
		return $rules;
	}
	
	public function login_registration_pages($pages) {
		if (w2dc_getValue($_GET, 'w2dc_listing_action') == 'add_review' || get_query_var('review-w2dc') == 'add-review') {
			$pages[] = array('id' => get_the_ID());
		}
	
		return $pages;
	}
	
	public function get_review_controller($controller) {
		global $w2dc_instance;
	
		if ($w2dc_instance->getShortcodeProperty(W2DC_REVIEW_PAGE_SHORTCODE)) {
			return $w2dc_instance->getShortcodeProperty(W2DC_REVIEW_PAGE_SHORTCODE);
		}
	
		if ($w2dc_instance->getShortcodeProperty(W2DC_ADD_REVIEW_PAGE_SHORTCODE)) {
			return $w2dc_instance->getShortcodeProperty(W2DC_ADD_REVIEW_PAGE_SHORTCODE);
		}
	
		return $controller;
	}
	
	public function do_process_directory_controller($do_process_directory_controller) {
		global $w2dc_instance;
	
		if (get_query_var('listing-w2dc') && get_query_var('review-w2dc')) {
			return true;
		}
	
		if (get_query_var('listing-w2dc') && get_query_var('review-action-w2dc') == 'all') {
			return true;
		}
	
		if (get_query_var('listing-w2dc') && (w2dc_getValue($_GET, 'w2dc_listing_action') == 'add_review' || get_query_var('review-action-w2dc') == 'new')) {
			return true;
		}
	
		return $do_process_directory_controller;
	}
	
	public function setup_breadcrumbs(&$controller) {
		$args = array(
				'post_type' => W2DC_POST_TYPE,
				'name' => get_query_var('listing-w2dc'),
				'posts_per_page' => 1,
		);
		$frontend_controller = new w2dc_frontend_controller;
		add_filter('post_limits', array($frontend_controller, 'findOnlyOnePost'));
		$posts = get_posts($args);
		remove_filter('post_limits', array($frontend_controller, 'findOnlyOnePost'));
		$listing_post = array_shift($posts);
		$listing = w2dc_getListing($listing_post);
			
		if (get_option('w2dc_enable_breadcrumbs')) {
			switch (get_option('w2dc_breadcrumbs_mode')) {
				case 'category':
					if ($terms = get_the_terms($listing->post->ID, W2DC_CATEGORIES_TAX)) {
						if (!empty($_GET['term_number']) && isset($terms[$_GET['term_number']-1]) && get_option('w2dc_permalinks_structure') == 'category_slug') {
							$term = $terms[$_GET['term_number']-1];
						} else {
							$term = array_shift($terms);
						}
						$controller->addBreadcrumbs(w2dc_get_term_parents($term, W2DC_CATEGORIES_TAX, true, true));
					}
					break;
				case 'location':
					if ($terms = get_the_terms($listing->post->ID, W2DC_LOCATIONS_TAX)) {
						if (!empty($_GET['term_number']) && isset($terms[$_GET['term_number']-1]) && get_option('w2dc_permalinks_structure') == 'location_slug') {
							$term = $terms[$_GET['term_number']-1];
						} else {
							$term = array_shift($terms);
						}
						$controller->addBreadcrumbs(w2dc_get_term_parents($term, W2DC_LOCATIONS_TAX, true, true));
					}
					break;
			}
			$listing_url = get_the_permalink($listing->post->ID);
			$controller->addBreadcrumbs(new w2dc_breadcrumb($listing->title(), rtrim($listing_url, '/')));
			$controller->addBreadcrumbs(new w2dc_breadcrumb(esc_html__('Reviews', 'W2DC'), w2dc_get_all_reviews_link($listing)));
		}
	
		return $listing;
	}
	
	public function directory_controller_construct($controller) {
		global $w2dc_instance;
	
		//var_dump(get_query_var('review-w2dc'));
		//var_dump(get_query_var('review-action-w2dc'));
		//var_dump(get_query_var('listing-w2dc'));
	
		if (get_query_var('listing-w2dc') && get_query_var('review-w2dc')) {
			$reviews_controller = new w2dc_reviews_controller();
			add_filter('post_limits', array($reviews_controller, 'findOnlyOnePost'));
			$reviews_controller->init(array(
					'name' => get_query_var('review-w2dc'),
			));
			remove_filter('post_limits', array($reviews_controller, 'findOnlyOnePost'));
			if (count($reviews_controller->reviews)) {
				$reviews_array = $reviews_controller->reviews;
				$review = array_shift($reviews_array);
	
				$controller->page_title = $review->title();
				$controller->review = $review;
				$controller->is_review = true;
				$controller->is_single = true;
				$controller->object_single = $review;
				$controller->template = array(W2DC_RATINGS_TEMPLATES_PATH, 'review_single.tpl.php');
				$listing = $this->setup_breadcrumbs($controller);
				$controller->addBreadCrumbs($review->title());
				$controller->add_template_args(array('reviews_controller' => $reviews_controller, 'listing' => $listing));
	
				if (!wp_doing_ajax()) {
					$controller->review->increaseClicksStats();
				}
	
				add_action('wp_head', array($controller, 'change_global_post'), -1000);
				add_action('wp_head', array($controller, 'back_global_post'), 1000);
				add_action('wp_head', array($controller, 'insert_fb_in_head'), -10);
				if (function_exists('rel_canonical')) {
					remove_action('wp_head', 'rel_canonical');
				}
				// replace the default WordPress canonical URL function with your own
				add_action('wp_head', array($controller, 'rel_canonical_with_custom_tag_override'));
	
				add_filter('w2dc_include_lightbox_files', '__return_true');
			} else {
				$controller->set404();
			}
		} elseif (get_query_var('listing-w2dc') && get_query_var('review-action-w2dc') == 'all') {
			$listing = $this->setup_breadcrumbs($controller);
			array_pop($controller->breadcrumbs);
			$controller->is_directory_page = true;
			$controller->addBreadcrumbs(esc_html__('Reviews', 'W2DC'));
			$controller->page_title = __('Reviews:', 'W2DC') . ' ' . $listing->title();
				
			$reviews_controller = new w2dc_reviews_controller;
			$reviews_controller->init(array(
					'listings' => array($listing->post->ID),
					'base_url' => w2dc_get_all_reviews_link($listing),
			));
			$controller->add_template_args(array('reviews_controller' => $reviews_controller, 'listing' => $listing));
			$controller->template = array(W2DC_RATINGS_TEMPLATES_PATH, 'reviews_page.tpl.php');
			
			$controller->base_url = w2dc_get_all_reviews_link($listing);
		} elseif (get_query_var('listing-w2dc') && (w2dc_getValue($_GET, 'w2dc_listing_action') == 'add_review' || get_query_var('review-action-w2dc') == 'new')) {
			$controller->is_directory_page = true;
			$controller->page_title = __('Add review', 'W2DC');
			$listing = $this->setup_breadcrumbs($controller);
			$controller->addBreadcrumbs(esc_html__('Add review', 'W2DC'));
			
			if (get_option('w2dc_reviews_allowed_users') == 'login' && !is_user_logged_in()) {
				if (get_option('w2dc_fsubmit_addon')) {
					$login_registrations = new w2dc_login_registrations;
					if ($login_registrations->is_action()) {
						$controller->template = $login_registrations->process($this);
					} else {
						if (w2dc_get_wpml_dependent_option('w2dc_submit_login_page') && w2dc_get_wpml_dependent_option('w2dc_submit_login_page') != get_the_ID()) {
							$url = get_permalink(w2dc_get_wpml_dependent_option('w2dc_submit_login_page'));
							$url = add_query_arg('redirect_to', urlencode(w2dc_get_add_review_link(get_the_ID())), $url);
							wp_redirect($url);
						} else {
							$login_registrations = new w2dc_login_registrations;
							$controller->template = $login_registrations->login_template();
						}
					}
				} else {
					$controller->template = array(W2DC_FSUBMIT_TEMPLATES_PATH, 'login_form.tpl.php');
				}
			} else {
				if (w2dc_can_user_add_review($listing->post->ID)) {
					$controller->w2dc_user_contact_name = '';
					$controller->w2dc_user_contact_email = '';
					if (!isset($_POST['review_id']) || !isset($_POST['review_id_hash']) || !is_numeric($_POST['review_id']) || md5($_POST['review_id'] . wp_salt()) != $_POST['review_id_hash']) {
						// Create Auto-Draft
						$new_post_args = array(
								'post_title' => __('Auto Draft', 'W2DC'),
								'post_type' => W2DC_REVIEW_TYPE,
								'post_status' => 'auto-draft'
						);
						if ($new_post_id = wp_insert_post($new_post_args)) {
							$this->saveInitialDraft($new_post_id, $listing);
		
							$review = w2dc_getReview($new_post_id);
		
							$w2dc_instance->media_manager->load_params(array(
									'object_id' => $review->post->ID,
									'images_number' => get_option("w2dc_reviews_images_number"),
									'videos_number' => 0,
									'logo_enabled' => false,
							));
						}
					} elseif (isset($_POST['submit']) && (isset($_POST['_submit_nonce']) && wp_verify_nonce($_POST['_submit_nonce'], 'w2dc_submit'))) {
						// This is existed Auto-Draft
						$review_id = $_POST['review_id'];
						$review = w2dc_getReview($review_id);
							
						if (get_option("w2dc_reviews_images_number") > 0) {
							$w2dc_instance->media_manager->load_media(array(
									'images' => $review->images,
							));
							$w2dc_instance->media_manager->load_params(array(
									'object_id' => $review_id,
									'images_number' => get_option("w2dc_reviews_images_number"),
									'videos_number' => 0,
									'logo_enabled' => false,
							));
						}
							
						$errors = array();
							
						if (!is_user_logged_in() && get_option('w2dc_reviews_allowed_users') == 'guests') {
							$w2dc_form_validation = new w2dc_form_validation();
							$w2dc_form_validation->set_rules('w2dc_user_contact_name', __('Contact Name', 'W2DC'), 'required');
							$w2dc_form_validation->set_rules('w2dc_user_contact_email', __('Contact Email', 'W2DC'), 'valid_email|required');
							if (!$w2dc_form_validation->run()) {
								$errors[] = $w2dc_form_validation->error_array();
							}
								
							$controller->w2dc_user_contact_name = $w2dc_form_validation->result_array('w2dc_user_contact_name');
							$controller->w2dc_user_contact_email = $w2dc_form_validation->result_array('w2dc_user_contact_email');
						}
							
						if (!isset($_POST['post_title']) || !trim($_POST['post_title']) || $_POST['post_title'] == __('Auto Draft', 'W2DC')) {
							$errors[] = __('Review title field required', 'W2DC');
							$post_title = __('Auto Draft', 'W2DC');
						} else {
							$post_title = trim($_POST['post_title']);
						}
							
						if (get_option("w2dc_reviews_images_number") > 0) {
							if ($validation_results = $w2dc_instance->media_manager->validateAttachments($errors)) {
								$w2dc_instance->media_manager->saveAttachments($validation_results);
							}
						}
							
						if (!w2dc_is_recaptcha_passed()) {
							$errors[] = esc_attr__("Anti-bot test wasn't passed!", 'W2DC');
						}
						
						// adapted for WPML
						global $sitepress;
						if (
						(
								(function_exists('wpml_object_id_filter') && $sitepress && $sitepress->get_default_language() != ICL_LANGUAGE_CODE && ($tos_page = get_option('w2dc_tospage_'.ICL_LANGUAGE_CODE)))
								||
								($tos_page = get_option('w2dc_tospage'))
						)
						&&
						(!isset($_POST['w2dc_tospage']) || !$_POST['w2dc_tospage'])
						) {
							$errors[] = __('Please check the box to agree the Terms of Services.', 'W2DC');
						}
						
						$this->saveReviewRatings($review_id);
							
						if ($errors) {
							$postarr = array(
									'ID' => $review_id,
									'post_title' => apply_filters('w2dc_title_save_pre', $post_title, $review),
									'post_name' => apply_filters('w2dc_name_save_pre', '', $review),
									'post_content' => (isset($_POST['post_content']) ? $_POST['post_content'] : ''),
									'post_excerpt' => (isset($_POST['post_excerpt']) ? $_POST['post_excerpt'] : ''),
									'post_type' => W2DC_REVIEW_TYPE,
							);
							$result = wp_update_post($postarr, true);
							if (is_wp_error($result)) {
								$errors[] = $result->get_error_message();
							}
								
							foreach ($errors AS $error) {
								w2dc_addMessage($error, 'error');
							}
						} else {
							if (!is_user_logged_in() && get_option('w2dc_reviews_allowed_users') == 'guests') {
								if (email_exists($this->w2dc_user_contact_email)) {
									$user = get_user_by('email', $this->w2dc_user_contact_email);
									$post_author_id = $user->ID;
									$post_author_username = $user->user_login;
								} else {
									$user_contact_name = trim($this->w2dc_user_contact_name);
									if ($user_contact_name) {
										$display_author_name = $user_contact_name;
										if (get_user_by('login', $user_contact_name))
											$login_author_name = $user_contact_name . '_' . time();
										else
											$login_author_name = $user_contact_name;
									} else {
										$display_author_name = 'Author_' . time();
										$login_author_name = 'Author_' . time();
									}
									if ($this->w2dc_user_contact_email) {
										$author_email = $this->w2dc_user_contact_email;
									} else {
										$author_email = '';
									}
		
									$password = wp_generate_password(6, false);
		
									$post_author_id = wp_insert_user(array(
											'display_name' => $display_author_name,
											'user_login' => $login_author_name,
											'user_email' => $author_email,
											'user_pass' => $password
									));
									$post_author_username = $login_author_name;
		
									if (!is_wp_error($post_author_id) && $author_email) {
										// WP auto-login
										wp_set_current_user($post_author_id);
										wp_set_auth_cookie($post_author_id);
										do_action('wp_login', $post_author_username, get_userdata($post_author_id));
		
										if (get_option('w2dc_newuser_notification')) {
											$subject = __('Registration notification', 'W2DC');
											$body = str_replace('[author]', $display_author_name,
													str_replace('[listing]', $post_title,
													str_replace('[login]', $login_author_name,
													str_replace('[password]', $password,
													get_option('w2dc_newuser_notification')))));
		
											if (w2dc_mail($author_email, $subject, $body)) {
												w2dc_addMessage(__('New user was created and added to the site, login and password were sent to provided contact email.', 'W2DC'));
											}
										}
									}
								}
							} elseif (is_user_logged_in()) {
								$post_author_id = get_current_user_id();
							} else {
								$post_author_id = 0;
							}
		
							if (get_option('w2dc_reviews_moderation')) {
								$post_status = 'pending';
								$message = esc_attr__("Review was saved successfully! Now it's awaiting moderators approval.", 'W2DC');
								update_post_meta($review_id, '_requires_moderation', true);
							} else {
								$post_status = 'publish';
								$message = __('Review was saved successfully! Now you can manage it in your dashboard.', 'W2DC');
							}
		
							$postarr = array(
									'ID' => $review_id,
									'post_title' => apply_filters('w2dc_title_save_pre', $post_title, $review),
									'post_name' => apply_filters('w2dc_name_save_pre', '', $review),
									'post_content' => (isset($_POST['post_content']) ? $_POST['post_content'] : ''),
									'post_excerpt' => (isset($_POST['post_excerpt']) ? $_POST['post_excerpt'] : ''),
									'post_type' => W2DC_REVIEW_TYPE,
									'post_author' => $post_author_id,
									'post_status' => $post_status
							);
							$result = wp_update_post($postarr, true);
							if (is_wp_error($result)) {
								w2dc_addMessage($result->get_error_message(), 'error');
							} else {
								if ($post_status == 'publish') {
									$this->saveAllRatings($review_id);
								}
								$this->saveReviewMeta($review_id);
		
								w2dc_addMessage($message);
		
								if (get_option('w2dc_newreview_admin_notification')) {
									$author = get_userdata($review->post->post_author);
		
									$subject = __('Notification about new review creation (do not reply)', 'W2DC');
									$body = str_replace('[user]', $author->display_name,
											str_replace('[review]', $post_title,
											str_replace('[link]', admin_url('post.php?post='.$review->post->ID.'&action=edit'),
											get_option('w2dc_newreview_admin_notification'))));
		
									w2dc_mail(w2dc_getAdminNotificationEmail(), $subject, $body);
								}
		
								apply_filters('w2dc_review_creation_front', $review);
		
								if (get_option('w2dc_fsubmit_addon') && !empty($w2dc_instance->dashboard_page_url)) {
									$redirect_to = add_query_arg('w2dc_action', 'reviews', w2dc_dashboardUrl());
								} else {
									$redirect_to = get_permalink($review->post);
								}
		
								$redirect_to = apply_filters('w2dc_redirect_after_review_submit', $redirect_to);
		
								wp_redirect($redirect_to);
								die();
							}
						}
						
						// renew data inside $review object
						$review = w2dc_getReview($review_id);
					}
		
					if (get_current_user_id()) {
						$current_user = wp_get_current_user();
						w2dc_addMessage(sprintf(__("You are logged in as %s. <a href='%s'>Log out</a> or continue submission in this account.", "W2DC"), $current_user->display_name, wp_logout_url()));
					} elseif (get_option('w2dc_reviews_allowed_users') == 'guests') {
						w2dc_addMessage(sprintf(__("Returning user? Please <a href='%s'>Log in</a> or register in this submission form.", "W2DC"), wp_login_url()));
					}
		
					$controller->add_template_args(array(
							'review' => $review,
							'listing' => $listing,
					));
		
					$controller->template = array(W2DC_RATINGS_TEMPLATES_PATH, 'review_add.tpl.php');
		
					if (get_option("w2dc_reviews_images_number") > 0) {
						add_action('wp_enqueue_scripts', array($w2dc_instance->media_manager, 'admin_enqueue_scripts_styles'));
					}
				} else {
					$controller->add_template_args(array(
							'listing' => $listing,
					));
					
					$controller->template = array(W2DC_RATINGS_TEMPLATES_PATH, 'review_add.tpl.php');
					
					w2dc_addMessage(sprintf(__("You are not allowed to create more than %d review(s) for one listing.", "W2DC"), get_option('w2dc_reviews_number_allowed')));
				}
			}
		}
	
		return $controller;
	}
	
	public function on_review_approval($new_status, $old_status, $post) {
		global $w2dc_instance;
		
		if (get_option('w2dc_approval_review_notification')) {
			if (
			$post->post_type == W2DC_REVIEW_TYPE &&
			'publish' == $new_status &&
			'pending' == $old_status &&
			($review = w2dc_getReview($post)) &&
			($author = get_userdata($review->post->post_author))
			) {
				update_post_meta($post->ID, '_review_approved', true);
	
				$subject = __('Approval of review', 'W2DC');
	
				$link = '';
				if (get_option('w2dc_fsubmit_addon') && !empty($w2dc_instance->dashboard_page_url)) {
					$link = w2dc_dashboardUrl();
				}
					
				$body = str_replace('[author]', $author->display_name,
						str_replace('[review]', $review->post->post_title,
						str_replace('[link]', $link,
						get_option('w2dc_approval_review_notification'))));
	
				w2dc_mail($author->user_email, $subject, $body);
			}
		}
	}
	
	public function review_vote() {
		$post_id = w2dc_getValue($_POST, 'post_id');
		$vote = w2dc_getValue($_POST, 'vote');
		$_wpnonce = wp_verify_nonce(w2dc_getValue($_POST, '_wpnonce'), 'review_vote');
	
		if (($post = get_post($post_id)) && ($vote == 'up' || $vote == 'down') && $_wpnonce) {
			$vote_num = ($vote == 'up') ? 1 : 0;
				
			$user_id = get_current_user_id();
			$ip = w2dc_ip_address();
			/* if (get_option('w2dc_only_registered_users') && !$user_id)
			 return false; */
	
			if (!$this->is_review_voted($post->ID)) {
				if ($user_id) {
					add_post_meta($post->ID, W2DC_REVIEW_VOTE_PREFIX . $user_id, $vote_num);
				} elseif ($ip) {
					add_post_meta($post->ID, W2DC_REVIEW_VOTE_PREFIX . $ip, $vote_num);
				}
	
				setcookie(W2DC_REVIEW_VOTE_PREFIX . $post->ID, $vote_num, time() + 31536000, '/');
			} else {
				// possible to change review vote
				if ($user_id) {
					update_post_meta($post->ID, W2DC_REVIEW_VOTE_PREFIX . $user_id, $vote_num);
				} elseif ($ip) {
					update_post_meta($post->ID, W2DC_REVIEW_VOTE_PREFIX . $ip, $vote_num);
				}
	
				setcookie(W2DC_REVIEW_VOTE_PREFIX . $post->ID, $vote_num, time() + 31536000, '/');
			}
				
			$review = new w2dc_review;
			$review->loadReviewFromPost($post);
			
			update_post_meta($post->ID, '_votes_sum', $review->votes_up - $review->votes_down);
			
			echo json_encode(array('votes_up' => $review->votes_up, 'votes_down' => $review->votes_down));
		}
		die();
	}
	
	public function is_review_voted($id) {
		if (!isset($_COOKIE[W2DC_REVIEW_VOTE_PREFIX . $id])) {
			if ($user_id = get_current_user_id()) {
				$meta = get_post_meta($id, W2DC_REVIEW_VOTE_PREFIX . $user_id, true);
				if ($meta != '') {
					return true;
				}
			}
	
			if ($ip = w2dc_ip_address()) {
				$meta = get_post_meta($id, W2DC_REVIEW_VOTE_PREFIX . $ip, true);
				if ($meta != '') {
					return true;
				}
			}
		} else {
			return true;
		}
	}
	
	public function add_reviews_dashboard_link($frontend_controller) {
		global $w2dc_instance, $wpdb;
		
		$current_user = wp_get_current_user();
		
		$reviews_counter = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->posts} AS p WHERE p.post_type='" . W2DC_REVIEW_TYPE . "' AND p.post_status IN ('publish', 'pending') AND p.post_author={$current_user->ID}");
	
		echo '<li ' . (($frontend_controller->active_tab == 'reviews') ? 'class="w2dc-active"' : '') . '><a href="' . w2dc_dashboardUrl(array('w2dc_action' => 'reviews')) . '">' . __('Reviews', 'W2DC'). ' (' . $reviews_counter . ')</a></li>';
	}
	
	public function handle_dashboard_controller($frontend_controller) {
		global $w2dc_instance;
	
		if (get_class($frontend_controller) == 'w2dc_dashboard_controller') {
			if ($w2dc_instance->action == 'reviews') {
				$current_user = wp_get_current_user();
				
				$reviews_controller = new w2dc_reviews_controller;
				$args = array(
						'author' => $current_user->ID,
						'status' => 'publish, pending, draft', // Any status
				);
				$reviews_controller->init($args);
				
				$frontend_controller->reviews = array();
				while ($reviews_controller->query->have_posts()) {
					$reviews_controller->query->the_post();
	
					$review = w2dc_getReview(get_the_ID());
					$frontend_controller->reviews[get_the_ID()] = $review;
				}
				// this is reset is really required after the loop ends
				wp_reset_postdata();
				
				$frontend_controller->add_template_args(array('reviews_controller' => $reviews_controller));
				$frontend_controller->template = W2DC_FSUBMIT_TEMPLATES_PATH . 'dashboard.tpl.php';
				$frontend_controller->subtemplate = W2DC_RATINGS_TEMPLATES_PATH . 'reviews_dashboard.tpl.php';
				$frontend_controller->active_tab = 'reviews';
			} elseif ($w2dc_instance->action == 'edit_review' && isset($_GET['review_id']) && is_numeric($_GET['review_id']) && w2dc_current_user_can_edit_review($_GET['review_id'])) {
				if (isset($_POST['referer'])) {
					$this->referer = $_POST['referer'];
				} else {
					$this->referer = wp_get_referer();
				}
				if (isset($_POST['cancel']) && isset($_POST['referer'])) {
					wp_redirect($_POST['referer']);
					die();
				}
				
				$review_id = $_GET['review_id'];
				
				if ($review = w2dc_getReview($review_id)) {
					if (w2dc_current_user_can_edit_review($review->post->ID)) {
						if (get_option("w2dc_reviews_images_number") > 0) {
							$w2dc_instance->media_manager->load_media(array(
									'images' => $review->images,
							));
							$w2dc_instance->media_manager->load_params(array(
									'object_id' => $review_id,
									'images_number' => get_option("w2dc_reviews_images_number"),
									'videos_number' => 0,
									'logo_enabled' => false,
							));
						}
							
						if (isset($_POST['submit'])) {
							$errors = array();
							
							if (!isset($_POST['post_title']) || !trim($_POST['post_title']) || $_POST['post_title'] == __('Auto Draft', 'W2DC')) {
								$errors[] = __('Review title field required', 'W2DC');
								$post_title = __('Auto Draft', 'W2DC');
							} else {
								$post_title = trim($_POST['post_title']);
							}
							
							if (get_option("w2dc_reviews_images_number") > 0) {
								if ($validation_results = $w2dc_instance->media_manager->validateAttachments($errors)) {
									$w2dc_instance->media_manager->saveAttachments($validation_results);
								}
							}
							
							$this->saveReviewRatings($review_id);
							
							if ($errors) {
								$postarr = array(
										'ID' => $review_id,
										'post_title' => apply_filters('w2dc_title_save_pre', $post_title, $review),
										'post_name' => apply_filters('w2dc_name_save_pre', '', $review),
										'post_content' => (isset($_POST['post_content']) ? $_POST['post_content'] : ''),
										'post_excerpt' => (isset($_POST['post_excerpt']) ? $_POST['post_excerpt'] : ''),
										'post_type' => W2DC_REVIEW_TYPE,
								);
								$result = wp_update_post($postarr, true);
								if (is_wp_error($result)) {
									$errors[] = $result->get_error_message();
								}
									
								foreach ($errors AS $error) {
									w2dc_addMessage($error, 'error');
								}
							} else {
								if ($review->post->post_status == 'publish') {
									if (get_option('w2dc_edit_reviews_moderation')) {
										$post_status = 'pending';
										update_post_meta($review_id, '_requires_moderation', true);
									} else {
										$post_status = 'publish';
									}
								}
								if (get_option('w2dc_edit_reviews_moderation')) {
									$message = esc_attr__("Review was saved successfully! Now it's awaiting moderators approval.", 'W2DC');
								} else {
									$message = __('Review was saved successfully! Now you can manage it in your dashboard.', 'W2DC');
								}
							
								$postarr = array(
										'ID' => $review_id,
										'post_title' => apply_filters('w2dc_title_save_pre', $post_title, $review),
										'post_name' => apply_filters('w2dc_name_save_pre', '', $review),
										'post_content' => (isset($_POST['post_content']) ? $_POST['post_content'] : ''),
										'post_excerpt' => (isset($_POST['post_excerpt']) ? $_POST['post_excerpt'] : '')
								);
								if (isset($post_status)) {
									$postarr['post_status'] = $post_status;
								}
							
								$result = wp_update_post($postarr, true);
								if (is_wp_error($result)) {
									w2dc_addMessage($result->get_error_message(), 'error');
								} else {
									$this->saveAllRatings($review_id);
									
									w2dc_addMessage($message);
							
									if (get_option('w2dc_editreview_admin_notification')) {
										// When listing was published and became pending after modification
										if ($review->post->post_status == 'publish' && $post_status == 'pending') {
											$author = wp_get_current_user();
							
											$subject = __('Notification about review modification (do not reply)', 'W2DC');
											$body = str_replace('[user]', $author->display_name,
													str_replace('[review]', $review->title(),
													str_replace('[link]', admin_url('post.php?post='.$review->post->ID.'&action=edit'),
													get_option('w2dc_editreview_admin_notification'))));
													
											w2dc_mail(w2dc_getAdminNotificationEmail(), $subject, $body);
										}
									}
									
									if ($post_title != $review->title()) {
										$url = get_the_permalink($review_id);
									} elseif ($this->referer) {
										$url = $this->referer;
									} else {
										$url = w2dc_dashboardUrl(array('w2dc_action' => 'reviews'));
									}
									
									wp_redirect($url);
									die();
								}
							}
							
							// renew data inside $review object
							$review = w2dc_getReview($review_id);
						}
					} else {
						wp_die('You are not able to manage this review', 'W2DC');
					}
					
					if (get_option("w2dc_reviews_images_number") > 0) {
						add_action('wp_enqueue_scripts', array($w2dc_instance->media_manager, 'admin_enqueue_scripts_styles'));
					}
	
					$frontend_controller->add_template_args(array('review' => $review, 'listing_info' => true, 'info_template' => array(W2DC_RATINGS_TEMPLATES_PATH, 'info_metabox.tpl.php')));
					$frontend_controller->template = W2DC_FSUBMIT_TEMPLATES_PATH . 'dashboard.tpl.php';
					$frontend_controller->subtemplate = W2DC_RATINGS_TEMPLATES_PATH . 'edit_review_dashboard.tpl.php';
					$frontend_controller->active_tab = 'reviews';
				}
			} elseif ($w2dc_instance->action == 'view_review_stats' && isset($_GET['review_id']) && is_numeric($_GET['review_id']) && w2dc_current_user_can_edit_review($_GET['review_id'])) {
				$review_id = $_GET['review_id'];
				
				if (get_option('w2dc_enable_stats') && w2dc_current_user_can_edit_listing($review_id) && ($review = w2dc_getReview($review_id))) {
					add_action('wp_enqueue_scripts', array($frontend_controller, 'enqueue_scripts_styles'));
					$frontend_controller->add_template_args(array('object' => $review, 'listing_info' => true, 'info_template' => array(W2DC_RATINGS_TEMPLATES_PATH, 'info_metabox.tpl.php')));
					$frontend_controller->template = array(W2DC_FSUBMIT_TEMPLATES_PATH, 'dashboard.tpl.php');
					$frontend_controller->subtemplate = array(W2DC_FSUBMIT_TEMPLATES_PATH, 'view_stats.tpl.php');
					$frontend_controller->active_tab = 'reviews';
				} else {
					wp_die('You are not able to manage this review', 'W2DC');
				}
			}
		}
	
		return $frontend_controller;
	}
	
	public function edit_review_link($url, $post_id) {
		global $w2dc_instance;
	
		if (!is_admin() && isset($w2dc_instance->dashboard_page_url) && ($post = get_post($post_id)) && $post->post_type == W2DC_REVIEW_TYPE) {
			return w2dc_dashboardUrl(array('w2dc_action' => 'edit_review', 'review_id' => $post_id));
		}
	
		return $url;
	}
	
	public function controller_request() {
		global $w2dc_instance;
		
		$post_args = $_POST;
		
		$hash = $_POST['hash'];
		
		$shortcode_atts = array_merge(array(
				'listings' => '',
				'name' => '',
				'perpage' => 10,
				'onepage' => 0,
				'reviews_order_by' => 'post_date',
				'reviews_order' => 'DESC',
				'hide_order' => 0,
				'hide_count' => 0,
				'hide_paginator' => 0,
				'show_views_switcher' => 1,
				'reviews_view_type' => 'list',
				'reviews_view_grid_columns' => 2,
				'reviews_thumb_width' => (int)get_option('w2dc_listing_thumb_width'),
				'wrap_logo_list_view' => 0,
				'logo_animation_effect' => (int)get_option('w2dc_logo_animation_effect'),
				'hide_content' => 0,
				'summary_on_logo_hover' => 0,
				'author' => 0,
				'paged' => 1,
				'scrolling_paginator' => 0,
				'grid_view_logo_ratio' => get_option('w2dc_grid_view_logo_ratio'), // 100 (1:1), 75 (4:3), 56.25 (16:9), 50 (2:1)
				'include_get_params' => 1,
				'template' => array(W2DC_RATINGS_TEMPLATES_PATH, 'reviews_block.tpl.php'),
		), $post_args);
		
		// This is required workaround
		$_REQUEST['reviews_order_by'] = w2dc_getValue($post_args, 'reviews_order_by', $shortcode_atts['reviews_order_by']);
		$_REQUEST['reviews_order'] = w2dc_getValue($post_args, 'reviews_order', $shortcode_atts['reviews_order']);
		
		// Strongly required for paginator
		set_query_var('page', $shortcode_atts['paged']);
		
		$controller = new w2dc_reviews_controller;
		$controller->init($shortcode_atts);
		$controller->hash = $hash;
		
		// this does not allow paginator to set up admin_url('admin-ajax.php') in get_pagenum_link()
		global $w2dc_global_base_url;
		$w2dc_global_base_url = $controller->base_url;
		add_filter('get_pagenum_link', array($w2dc_instance->ajax_controller, 'get_pagenum_link'));
		
		$reviews_html = w2dc_renderTemplate(array(W2DC_RATINGS_TEMPLATES_PATH, 'reviews_block.tpl.php'), array('frontend_controller' => $controller), true);

		wp_reset_postdata();
		
		$out = array(
				'html' => $reviews_html,
				'hash' => $controller->hash,
				'hide_show_more_listings_button' => ($shortcode_atts['paged'] >= $controller->query->max_num_pages) ? 1 : 0,
		);
		
		if ($json = json_encode(w2dc_utf8ize($out))) {
			echo $json;
		} else {
			echo json_last_error_msg();
		}
		
		die();
	}
}

?>