<?php

class w2dc_csv_import_export_reviews {
	public $csv_manager;
	public $collation_fields;
	public $export_rows_counter;
	public $total_rejected_lines;
	public $ratings_criterias;
	
	public function __construct($csv_manager) {
		$this->csv_manager = $csv_manager;
	}
	
	public function buildCollationColumns() {
		$this->csv_manager->collation_fields = array(
				'post_id' => __('Post ID* (existing review)', 'W2DC'),
				'listing_id' => __('Listing ID*', 'W2DC'),
				'title' => __('Title*', 'W2DC'),
				'user' => __('Author', 'W2DC'),
				'content' => __('Description', 'W2DC'),
				'images' => __('Images files', 'W2DC'),
				'creation_date' => __('Creation date', 'W2DC'),
				'ratings' => __('Rating or multi-ratings', 'W2DC'),
				'votes' => __('Votes', 'W2DC'),
		);
		
		$this->csv_manager->collation_fields = apply_filters('w2dc_csv_collation_fields_list', $this->csv_manager->collation_fields);
		
		if ($this->csv_manager->import_type == 'create_reviews') {
			unset($this->csv_manager->collation_fields['post_id']);
		}
		
		return $this->csv_manager->collation_fields;
	}
	
	public function validateSettings($validation) {
		$validation->set_rules('author', __('Reviews author', 'W2DC'), 'required|numeric');
	}
	
	public function buildSettings($validation) {
		$this->csv_manager->selected_user = $validation->result_array('author');
	}
	
	public function addTemplateFields($template_fields) {
		return array_merge($template_fields, array(
				'author' => $this->csv_manager->selected_user,
		));
	}
	
	public function csvImport() {
		if ($this->csv_manager->import_type == 'update_reviews' && !in_array('post_id', $this->csv_manager->collated_fields)) {
			$this->csv_manager->log['errors'][] = esc_attr__("Post ID field wasn't collated", 'W2DC');
		}
		if ($this->csv_manager->import_type == 'create_reviews' && !in_array('title', $this->csv_manager->collated_fields)) {
			$this->csv_manager->log['errors'][] = esc_attr__("Title field wasn't collated", 'W2DC');
		}
		if ($this->csv_manager->import_type == 'create_reviews' && $this->csv_manager->selected_user != 0 && !get_userdata($this->csv_manager->selected_user)) {
			$this->csv_manager->log['errors'][] = esc_attr__("There isn't author user you selected", 'W2DC');
		}
		if ($this->csv_manager->import_type == 'create_reviews' && $this->csv_manager->selected_user == 0 && !in_array('user', $this->csv_manager->collated_fields)) {
			$this->csv_manager->log['errors'][] = esc_attr__("Author field wasn't collated and default author wasn't selected", 'W2DC');
		}
	}
	
	public function processCSVImport() {
		global $w2dc_instance;
		
		$this->ratings_criterias = $w2dc_instance->reviews_manager->getRatingsCriterias();
		
		$this->total_rejected_lines = 0;
		foreach ($this->csv_manager->rows as $line=>$row) {
			$n = $line+1;
			printf(__('Importing line %d...', 'W2DC'), $n);
			echo "<br />";
			$error_on_line = false;
			$review_data = array();
			foreach ($this->csv_manager->collated_fields as $i=>$field) {
				$value = htmlspecialchars_decode(trim($row[$i])); // htmlspecialchars_decode() needed due to &amp; symbols in import files, ';' symbols can break import
		
				if ($field == 'post_id' && $this->csv_manager->import_type == 'update_reviews') {
					if (($post = get_post($value)) && ($review = w2dc_getReview($post))) {
						$review_data['existing_review'] = $review;
						$review_data['post_id'] = $value;
					} else {
						$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("Review with ID \"%d\" doesn't exist", 'W2DC'), $n, $value);
						$error_on_line = $this->csv_manager->setErrorOnLine($error);
					}
				}
		
				if ($field == 'listing_id') {
					if ($listing_post = get_post($value)) {
						$review_data['listing_id'] = $value;
					} else {
						$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("Listing with ID \"%d\" doesn't exist", 'W2DC'), $n, $value);
						$error_on_line = $this->csv_manager->setErrorOnLine($error);
					}
				} elseif ($field == 'title') {
					$review_data['title'] = $value;
					printf(__('Review title: %s', 'W2DC'), $value);
					echo "<br />";
				} elseif ($field == 'user') {
					if (!$this->csv_manager->selected_user) {
						$user_info = explode('>', $value);
						if (is_array($user_info) && !is_numeric($user_info[0]) && filter_var($user_info[1], FILTER_VALIDATE_EMAIL) && ($key = array_search($user_info[1], $this->csv_manager->users_emails)) !== FALSE) {
							// if it is existing user with format user_name>user@email.com
							$review_data['user_id'] = $this->csv_manager->users_ids[$key];
						} elseif ((($key = array_search($value, $this->csv_manager->users_logins)) !== FALSE) || (($key = array_search($value, $this->csv_manager->users_emails)) !== FALSE) || (($key = array_search($value, $this->csv_manager->users_ids))) !== FALSE) {
							// if it is existing user by login, email or ID
							$review_data['user_id'] = $this->csv_manager->users_ids[$key];
						} else {
							// it is new user
							if (!is_numeric($user_info[0]) && filter_var($user_info[1], FILTER_VALIDATE_EMAIL)) {
								$review_data['user_info'] = $user_info;
							} else {
								$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("User \"%s\" doesn't exist and format does not allow to create new user", 'W2DC'), $n, $value);
								$error_on_line = $this->csv_manager->setErrorOnLine($error);
							}
						}
					} else {
						$review_data['user_id'] = $this->csv_manager->selected_user;
					}
				} elseif ($field == 'content') {
					$review_data['content'] = $value;
				} elseif ($field == 'images') {
					if ($this->csv_manager->images_dir) {
						$review_data['images'] = array_filter(array_map('trim', explode($this->csv_manager->values_separator, $value)));
					} else {
						$images_value = array_filter(array_map('trim', explode($this->csv_manager->values_separator, $value)));
						$validation = new w2dc_form_validation();
						$this_is_import_by_URL = false;
						foreach ($images_value AS $image_url) {
							if ($validation->valid_url($image_url, false)) {
								$review_data['images'][] = $image_url;
								$this_is_import_by_URL = true;
							} else {
								$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__(sprintf("Incorrect image URL %s", 'W2DC'), $image_url), $n);
								$error_on_line = $this->csv_manager->setErrorOnLine($error);
							}
						}
						if (!$this_is_import_by_URL) {
							$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("Images column was specified, but ZIP archive wasn't upload", 'W2DC'), $n);
							$error_on_line = $this->csv_manager->setErrorOnLine($error);
						}
					}
				} elseif ($field == 'creation_date') {
					if (!($timestamp = strtotime($value))) {
						$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("Creation date value is incorrect", 'W2DC'), $n);
						$error_on_line = $this->csv_manager->setErrorOnLine($error);
					} else
						$review_data['creation_date'] = $timestamp;
				} elseif ($field == 'ratings') {
					if (is_numeric($value)) {
						$review_data['avg_rating'] = $value;
					} else {
						if ($this->ratings_criterias) {
							$ratings = array_filter(array_map('trim', explode($this->csv_manager->values_separator, $value)));
							if ($ratings) {
								$review_data['ratings'] = $ratings;
							} else {
								$error = sprintf(__('Error on line %d: ', 'W2DC') . esc_attr__("Can not recognize ratings", 'W2DC'), $n);
								$error_on_line = $this->csv_manager->setErrorOnLine($error);
							}
						}
					}
				} elseif ($field == 'votes') {
					$review_data['votes'] = array_filter(array_map('trim', explode($this->csv_manager->values_separator, $value)));
				}
				
				$review_data = apply_filters('w2dc_csv_process_fields', $review_data, $field, $value);
			}
		
			if (!$error_on_line) {
				if (!$this->csv_manager->test_mode) {
					if ($this->csv_manager->import_type == 'create_reviews') {
						$new_post_args = array(
								'post_title' => $review_data['title'],
								'post_type' => W2DC_REVIEW_TYPE,
								'post_author' => $this->csv_manager->processUser($review_data, $n),
								'post_status' => 'publish',
								'post_content' => (isset($review_data['content']) ? $review_data['content'] : ''),
								'post_date' => (isset($review_data['creation_date']) ? date('Y-m-d H:i:s', $review_data['creation_date']) : ''),
						);
						$new_post_id = wp_insert_post($new_post_args);
						
						$w2dc_instance->reviews_manager->saveReviewMeta($new_post_id);
						
						if (isset($review_data['listing_id'])) {
							add_post_meta($new_post_id, '_listing_id', $review_data['listing_id']);
						}
						if (isset($review_data['images'])) {
							$this->csv_manager->processImages($new_post_id, $review_data, $n);
						}
						if (isset($review_data['avg_rating'])) {
							add_post_meta($new_post_id, '_avg_rating', $review_data['avg_rating']);
						}
						if (isset($review_data['ratings'])) {
							$this->processRatings($new_post_id, $review_data, $n);
						}
						if (isset($review_data['votes'])) {
							$this->processVotes($new_post_id, $review_data, $n);
						}
		
						do_action('w2dc_csv_create_review', $new_post_id, $review_data);
					} elseif ($this->csv_manager->import_type == 'update_reviews') {
						// -------------------- Update existing listing by ID ------------------------------------------------------------------------------------------------------------------
						$existing_post_id = $review_data['post_id'];
		
						$existing_post_args = array(
								'ID' => $existing_post_id,
						);
						if (isset($review_data['user_id']) || isset($review_data['user_info']) || $this->csv_manager->selected_user) {
							$existing_post_args['post_author'] = $this->csv_manager->processUser($review_data, $n);
						}
						if (isset($review_data['title'])) {
							$existing_post_args['post_title'] = $review_data['title'];
						}
						if (isset($review_data['content'])) {
							$existing_post_args['post_content'] = $review_data['content'];
						}
						if (isset($review_data['creation_date'])) {
							$existing_post_args['post_date'] = date('Y-m-d H:i:s', $review_data['creation_date']);
						}
						wp_update_post($existing_post_args);
		
						if (isset($review_data['listing_id'])) {
							update_post_meta($existing_post_id, '_listing_id', $review_data['listing_id']);
						}
						if (isset($review_data['images'])) {
							wp_delete_attachment($existing_post_id);
							delete_post_meta($existing_post_id, '_attached_image');
		
							$this->csv_manager->processImages($existing_post_id, $review_data, $n);
						}
						if (isset($review_data['avg_rating'])) {
							update_post_meta($existing_post_id, '_avg_rating', $review_data['avg_rating']);
						}
						if (isset($review_data['ratings'])) {
							$this->processRatings($existing_post_id, $review_data, $n);
						}
						if (isset($review_data['votes'])) {
							$this->processVotes($existing_post_id, $review_data, $n);
						}
		
						do_action('w2dc_csv_update_review', $existing_post_id, $review_data);
					}
					wp_cache_flush();
				}
			} else {
				$this->total_rejected_lines++;
			}
		}
	}
	
	public function processRatings($post_id, &$review_data, $line_n) {
		global $w2dc_ratings_instance;
		
		$review_ratings = $review_data['ratings'];
		
		$total_rating = 0;
		foreach ($this->ratings_criterias AS $key=>$criteria) {
			$total_rating += $review_data['ratings'][$key];
		}
		$avg_rating = ($total_rating/count($this->ratings_criterias)+1)/2;
		
		update_post_meta($post_id, '_review_ratings', $review_ratings);
		update_post_meta($post_id, '_avg_rating', $avg_rating);
		
		$listing_id = get_post_meta($post_id, '_listing_id', true);
		$ip = w2dc_ip_address();
		update_post_meta($listing_id, W2DC_RATING_PREFIX . $ip, $avg_rating);

		$avg_rating = new w2dc_avg_rating($listing_id);
		$avg_rating->update_avg_rating();
	}
	
	public function processVotes($post_id, &$review_data, $line_n) {
		global $wpdb;
		
		$like = $wpdb->esc_like(W2DC_REVIEW_VOTE_PREFIX);
		$wpdb->query($wpdb->prepare("DELETE FROM {$wpdb->postmeta} WHERE post_id=%d AND meta_key LIKE %s", $post_id, $like.'%'), ARRAY_A);
		
		$votes_up = 0;
		$votes_down = 0;
		foreach ($review_data['votes'] AS $vote) {
			$vote = explode('>', $vote);
			$vote_user = $vote[0];
			$vote_num = $vote[1];
			
			if ($vote_num == 1) {
				$votes_up++;
			} elseif ($vote_num == 0) {
				$votes_down++;
			}
			
			add_post_meta($post_id, W2DC_REVIEW_VOTE_PREFIX . $vote_user, $vote_num);
		}
		update_post_meta($post_id, '_votes_sum', $votes_up - $votes_down);
	}
	
	public function csvExport($number, $offset) {
		global $wpdb, $w2dc_instance;
		
		$ratings_criterias = $w2dc_instance->reviews_manager->getRatingsCriterias();
		
		$csv_columns = array(
				'post_id',
				'listing_id',
				'title',
				'user',
				'description',
				'images',
				'date',
				'ratings',
				'votes',
		);
		
		$csv_output[] = $csv_columns;
		
		$args = array(
				'post_type' => W2DC_REVIEW_TYPE,
				'orderby' => 'ID',
				'order' => 'ASC',
				'post_status' => 'publish,private,draft,pending',
				'posts_per_page' => $number,
				'offset' => $offset,
		);
		$query = new WP_Query($args);
		$this->export_rows_counter = 0;
		while ($query->have_posts()) {
			$this->export_rows_counter++;
			$query->the_post();
			$post = get_post();
			$review = w2dc_getReview($post);
				
			$review_id = $review->post->ID;
			
			$ratings = array();
			if ($ratings_criterias) {
				$ratings = implode(';', get_post_meta($review_id, '_review_ratings', true));
			} else {
				$ratings = $review->avg_rating;
			}
			
			$votes = array();
			$like = $wpdb->esc_like(W2DC_REVIEW_VOTE_PREFIX);
			$results = $wpdb->get_results($wpdb->prepare("SELECT DISTINCT meta_key, meta_value FROM {$wpdb->postmeta} WHERE post_id=%d AND meta_key LIKE %s", $review_id, $like.'%'), ARRAY_A);
			foreach ($results AS $row) {
				$votes[] = str_replace(W2DC_REVIEW_VOTE_PREFIX, '', $row['meta_key']) . '>' . $row['meta_value'];
			}
				
			$images = array();
			foreach ($review->images AS $attachment_id=>$image) {
				$image_src = wp_get_attachment_image_src($attachment_id, 'full');
				$image_item = basename($image_src[0]);
				if ($image['post_title']) {
					$image_item .= ">" . $image['post_title'];
				}
				$images[] = $image_item;
			}
		
			$row = array(
					$review_id,
					get_post_meta($review_id, '_listing_id', true),
					$review->title(),
					$review->post->post_author,
					$review->post->post_content,
					implode(';', $images),
					date('d.m.Y H:i', mysql2date('U', $review->post->post_date)),
					$ratings,
					implode(';', $votes),
			);
		
			$csv_output[] = $row;
		}
		
		return $csv_output;
	}
	
	public function csvExportFileName() {
		return 'w2dc-reviews--' . date('Y-m-d_H_i_s') . '--' . $this->export_rows_counter . '.csv';
	}
}
?>